;
;	>> Bob Ross - The Joy of Pixeling the Walls at the Disco <<
;
;	A 256 bytes intro for the Atari 800 XL by JAC! on 2016-10-25
;	Requires Atari XL OS REV C due to dirty nasty direct calls.
;	Released in the 256 bytes compo at SillyVenture 2016 in Gdansk.
;
;	This started long ago with the idea to generate pixel stuff in 1k
; 	when I saw some nice PC procedural graphics. The idea evolved into
;	the plan to generate the different planets of the solar system.
;	As it turned out with single byte precision, the radius of the
;	planets was too limited. So I looked for something that could be 
;	created out of smaller balls and ran into the "30 years Amiga juggler"
; 	post somewhere on Facebook. I was hooked and tried getting the
;	coordinates and animation right based on a great article about it,
;	but I soon realized that the rendering of the man alone with single
;	balls and some 8 animation frames would take hours. So nothing
; 	suitable for a party where everything is shown live on real hardware. 
;	Sillyventure came closer and closer and having just one week left
;	with nothing to release yet, I decided to go for the 256B compo
;	instead of the intro compo. I spent Sunday getting the shaded circle
;	routine running and designing the man. Monday, I took a day off for
;	crunching it from 489B to 256B. The biggest obstacle turned out to
;	be the size of the data for the man. 18 balls with 5 parameters
;	(crunched into 4 bytes each) left me with 178 bytes for the code.
;
;	The idea for the title and the way the man is drawn came up the day
;	before I actually started coding when I zapped into one of the great
;	"Bob Ross - The joy of painting" shows at 02:00 AM. I saw how he
;	painted layer on layer to the canvas to create the depth effect.
;	And there was this unique noise caused by the way he uses the brush,
;	so this had to be in somehow. Not to mention that the ball man head
;	comes quite close to Bob Ross' classic hairstyle.
;
;	Created using WUDSN IDE. Visit http://www.wudsn.com to see more.


os_graphics	= $ef9c	;Dirty
os_plot		= $f1a4	;Nasty

rowcrs		= $54	;Row of cursor, 1 byte
colcrs		= $55	;Column of cursor, 2 bytes

pos_x		= $80
pos_y		= $81

positions_zp	= $82	;4 consecutive bytes
org_x		= $82
org_y		= $83
square_limit	= $84
diameter	= $85

ball_color	= $86	;Computed from square_limit

radius		= $88	;Byte, used in circle

result		= $90	;Word, only low byte used
factor		= $92	;Word, only low byte used
square_offset	= $94	;Byte
square_tmp	= $95	;Byte

	.macro m_assert_same_page ;For timing critical loop or ranges enclosed by ".PROC" or ".LOCAL"
	.if :1 / $100 <> (:1 + .len :1 - 1) / $100
		.if .def alignment_mode
			.error ":1 crosses page boundary between ", :1, , " - ", :1 + .len :1 - 1
		.else
			.print ":1 crosses page boundary between ", :1, , " - ", :1 + .len :1 - 1
		.endif
	.else
		.print ":1 within page boundary between ", :1, " - ", :1 + .len :1 - 1
	.endif
	.endm

	org $4000	;Make sure the positions do not cross a page boudary

	lda #15		;Graphics 15 hires mode
	jsr os_graphics	;Illegal entry point in REV C OS, returns with X=15, C=1
	stx 709

main_loop
	.proc get_position
	ldx #4
loop	lda positions
index = loop+1
	bne no_stop

	.proc stop

	ldx $d40b
	bpl skip
	lda 20
	asl
	asl
	sta $d204-$9c,x
skip	stx $d01a
	jmp stop
	.end

no_stop	sta positions_zp-1,x
	inc index
	dex
	bne loop

	lsr square_limit		;Divide by 2, LSB into <C>
	txa				;<X>=0
	rol				;<C>*2
	sec				;ORA #1
	rol				;<C*2> ORA #1 => 1 or 3
	sta ball_color
	.endp
 
 	.proc circle

 	mva diameter pos_y
loop_y 	mva diameter pos_x
	lsr
 	sta radius

loop_x	lda pos_x
	lsr
	clc
	adc org_x
	sta colcrs

 	lda org_y
 	adc pos_y
 	sta rowcrs
 	adc colcrs
	sta $d201

 	lda radius
	jsr square_xy_with_offset

	cmp square_limit
	bcs skip

	lda radius		;<C>==0
	sbc #10
	jsr square_xy_with_offset

too_big	ldx $d20a
	cpx square_limit
	bcs too_big
	cpx square_tmp
	bcs white

	lda ball_color
	.byte $2c		;Use "bit ABS" to fall trough
white	lda #2
	jsr os_plot

skip	dec pos_x
	bne loop_x

	dec pos_y
	bne loop_y
	.endp			;End of circle

	beq main_loop


	.proc square_xy_with_offset
	sta square_offset
	lda pos_x
	jsr square_with_offset
	sta square_tmp
	lda pos_y
	jsr square_with_offset
	clc
	adc square_tmp
	sta square_tmp
	sta $d200
	rts

	.proc square_with_offset ;IN: <A>, square_offset, OUT: <A>

	sec			;Must be exact here
	sbc square_offset

	cmp #$80
	bcc not_neg
	eor #$ff
	adc #$00
not_neg
	asl
	asl
	sta factor
	tax
	ldy #0	
	sty result		;High byte is kept in <Y>
	tya			;Low byte is kept in <A>
loop
;	clc			;No need to be exact as long as we're deterministic
	adc factor
	bcc no_carry
	iny
no_carry
	dex
	bne loop
	tya
	rts
	.endp			;End of square_with_offset
 
	.endp			;End of square_xy_with_offset
 
	.local positions

	.macro m_ball
	.byte :3, (:4*2 + (:5/2)), :2, :1+31
	.endm

	m_ball 48,174,40,20,1	;Left shoe
	m_ball 75,180,40,20,1	;Right shoe
	
	m_ball 48,144,40,20,3	;Left lower leg
	m_ball 75,150,40,20,3	;Right lower leg

	m_ball 51,114,32,16,3	;Left leg
	m_ball 53,70,64,64,1	;Belly
	m_ball 75,120,32,16,3	;Right leg

	m_ball 19,20,16,5,1	;Left hand
	m_ball 21,30,28,14,3	;Left arm
	m_ball 28,50,30,15,3
	m_ball 41,60,32,16,3

	m_ball 53,50,64,64,1	;Chest

	m_ball 65,34,18,6,3	;Neck
	m_ball 57,1,48,32,1	;Head

	m_ball 80,65,32,16,3	;Right arm
	m_ball 92,80,34,17,3
	m_ball 100,100,34,17,3
	m_ball 106,126,16,5,1	;Right hand

;	.byte 0			;Assume the next byte is $00
	.endl

	m_assert_same_page positions